@@ -1,5 +1,5 @@
 (*  Wyrd -- a curses-based front-end for Remind
- *  Copyright (C) 2005, 2006 Paul Pelzl
+ *  Copyright (C) 2005, 2006, 2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -48,7 +48,10 @@ type reminder_type_t = Timed | Untimed | General of int
 let create_windows screen =
    let height, width  = get_size () in
    let cal_height     = 10
-   and cal_width      = 40 in
+   and cal_width      = if !Rcfile.untimed_window_width < 34 then 34
+                        else if !Rcfile.untimed_window_width > width - 40 then width - 40
+                        else !Rcfile.untimed_window_width
+                        in
    let msg_height     = 6 in
    let err_height     = 1 in
    let timed_height   = height - 1 - msg_height - err_height
@@ -91,7 +94,10 @@ let create_windows screen =
 let resize_subwins iface =
    let height, width  = get_size () in
    let cal_height     = 10
-   and cal_width      = 40 in
+   and cal_width      = if !Rcfile.untimed_window_width < 34 then 34
+                        else if !Rcfile.untimed_window_width > width - 40 then width - 40
+                        else 40
+                        in
    let msg_height     = 6 in
    let err_height     = 1 in
    let timed_height   = height - 1 - msg_height - err_height
@@ -135,6 +141,49 @@ let resize_subwins iface =
    new_scr
 
 
+(* Parse a natural language event description and append
+ * the result to the specified reminders file. *)
+let append_quick_event event_spec remfile =
+   let (rem_spec, description) = 
+      Time_lang.parse_natural_language_event event_spec
+   in
+   let remline =
+      begin match rem_spec with
+      | Time_lang.Timed (start, finish) ->
+         let date_s = 
+            Printf.sprintf "REM %s %d %d " (Utility.string_of_tm_mon start.Unix.tm_mon)
+            start.Unix.tm_mday (start.Unix.tm_year + 1900)
+         in
+         let start_s = 
+            Printf.sprintf "AT %.2d:%.2d " start.Unix.tm_hour start.Unix.tm_min
+         in
+         let duration_s =
+            if start <> finish then
+               let (start_ts, _)  = Unix.mktime start
+               and (finish_ts, _) = Unix.mktime finish in
+               let dur = int_of_float ((finish_ts -. start_ts) /. 60.0) in
+               Printf.sprintf "DURATION %d:%.2d " (dur / 60) (dur mod 60)
+            else
+               ""
+         in
+         date_s ^ start_s ^ duration_s ^ "MSG " ^ description ^ "\n"
+      | Time_lang.Untimed timespec ->
+         let date_s = 
+            Printf.sprintf "REM %s %d %d " (Utility.string_of_tm_mon timespec.Unix.tm_mon)
+            timespec.Unix.tm_mday (timespec.Unix.tm_year + 1900)
+         in
+         date_s ^ "MSG " ^ description ^ "\n"
+      end
+   in
+   (* append the remline to the reminders file *)
+   let remfile_channel = 
+      open_out_gen [Open_append; Open_creat; Open_text] 416 (Utility.expand_file remfile)
+   in
+   output_string remfile_channel remline;
+   close_out remfile_channel;
+   (rem_spec, remline)
+
+
 (* refresh the screen *)
 let handle_refresh (iface : interface_state_t) reminders =
    let _ = wtouchln iface.scr.help_win 0 1 true in
@@ -158,25 +207,44 @@ let handle_refresh (iface : interface_state_t) reminders =
 (* handle a curses resize *)
 let handle_resize (iface : interface_state_t) reminders =
    endwin ();
-   let new_scr = resize_subwins iface in
-   let resized_iface = {
-      iface with scr = new_scr;
-                 top_untimed = 0;
-                 left_selection = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 1;
-                 right_selection = 1;
-                 timed_lineinfo = Array.make new_scr.tw_lines []
-   } in
-   begin try
-      assert (curs_set 0)
-   with _ ->
-      ()
+   begin match iface.resize_failed_win with
+   | None -> ()
+   | Some w -> let _ = delwin w in ()
    end;
-   handle_refresh resized_iface reminders
+   begin try
+      let new_scr = resize_subwins iface in
+      let resized_iface = {
+         iface with scr               = new_scr;
+                    top_untimed       = 0;
+                    left_selection    = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 1;
+                    right_selection   = 1;
+                    timed_lineinfo    = Array.make new_scr.tw_lines [];
+                    resize_failed_win = None
+      } in
+      begin try
+         assert (curs_set 0)
+      with _ ->
+         ()
+      end;
+      clear ();
+      assert (refresh ());
+      handle_refresh resized_iface reminders
+   with Failure s ->
+      (* Create a new full-screen window where we can
+       * paint an error message. *)
+      let w = newwin 0 0 0 0 in
+      erase ();
+      assert (mvwaddstr w 0 0 s);
+      assert (wnoutrefresh w);
+      assert (doupdate ());
+      ({iface with resize_failed_win = Some w}, reminders)
+   end
+      
    
 
 (* Any time a new item is selected, the reminders
  * record needs updating and the screen need redrawing *)
-let handle_selection_change iface reminders =
+let handle_selection_change iface reminders track_home_setting =
    let new_reminders = Remind.update_reminders reminders 
    (timestamp_of_line iface iface.left_selection) in
    let new_iface = draw_untimed iface new_reminders.Remind.curr_untimed in
@@ -184,18 +252,18 @@ let handle_selection_change iface reminders =
    draw_date_strip new_iface;
    draw_calendar new_iface new_reminders;
    draw_error new_iface "" false;
-   (new_iface, new_reminders)
+   ({new_iface with track_home = track_home_setting}, new_reminders)
 
 
 (* Same as previous, but without calling draw_timed () or
  * draw_date_strip ().  Optimization for the scrolling case. *)
-let handle_selection_change_scroll iface reminders =
+let handle_selection_change_scroll iface reminders track_home_setting =
    let new_reminders = Remind.update_reminders reminders 
    (timestamp_of_line iface iface.left_selection) in
    draw_calendar iface new_reminders;
    let new_iface = draw_untimed iface new_reminders.Remind.curr_untimed in
    draw_error new_iface "" false;
-   (new_iface, new_reminders)
+   ({new_iface with track_home = track_home_setting}, new_reminders)
 
 
 (* handle a "scroll down" event when the timed window is focused
@@ -206,7 +274,7 @@ let handle_scrolldown_timed_center (iface : interface_state_t) reminders =
       iface with top_timestamp = second_timestamp
    } in
    let (new_iface, new_reminders) =
-      handle_selection_change_scroll iface2 reminders
+      handle_selection_change_scroll iface2 reminders false
    in
    (* use a curses scroll operation to shift up the timed window *)
    assert (wscrl new_iface.scr.timed_win 1);
@@ -233,7 +301,7 @@ let handle_scrolldown_timed_nocenter (iface : interface_state_t) reminders =
          iface with left_selection = succ iface.left_selection
       } in
       let (new_iface, new_reminders) = 
-         handle_selection_change_scroll iface2 reminders
+         handle_selection_change_scroll iface2 reminders false
       in
       (* make a two-line update to erase and redraw the cursor *)
       let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 
@@ -246,7 +314,7 @@ let handle_scrolldown_timed_nocenter (iface : interface_state_t) reminders =
          iface with top_timestamp = second_timestamp
       } in
       let (new_iface, new_reminders) = 
-         handle_selection_change_scroll iface2 reminders
+         handle_selection_change_scroll iface2 reminders false
       in
       (* use a curses scroll operation to shift up the timed window *)
       assert (wscrl new_iface.scr.timed_win 1);
@@ -293,7 +361,7 @@ let handle_scrolldown_untimed (iface : interface_state_t) reminders =
          let new_iface = {
             iface with right_selection = succ iface.right_selection
          } in
-         handle_selection_change new_iface reminders
+         handle_selection_change new_iface reminders false
       end else
          (iface, reminders)
    end else begin
@@ -302,7 +370,7 @@ let handle_scrolldown_untimed (iface : interface_state_t) reminders =
          let new_iface = {
             iface with top_untimed = succ iface.top_untimed
          } in
-         handle_selection_change new_iface reminders
+         handle_selection_change new_iface reminders false
       end else
          (iface, reminders)
    end
@@ -317,7 +385,7 @@ let handle_scrollup_timed_center (iface : interface_state_t) reminders =
       iface with top_timestamp = prev_timestamp
    } in
    let (new_iface, new_reminders) = 
-      handle_selection_change_scroll iface2 reminders
+      handle_selection_change_scroll iface2 reminders false
    in
    (* use a curses scroll operation to shift up the timed window *)
    assert (wscrl new_iface.scr.timed_win (-1));
@@ -343,7 +411,7 @@ let handle_scrollup_timed_nocenter (iface : interface_state_t) reminders =
          iface with left_selection = pred iface.left_selection
       } in
       let (new_iface, new_reminders) = 
-         handle_selection_change_scroll iface2 reminders
+         handle_selection_change_scroll iface2 reminders false
       in
       (* make a two-line update to erase and redraw the cursor *)
       let new_iface = draw_timed_try_window new_iface new_reminders.Remind.all_timed 
@@ -356,7 +424,7 @@ let handle_scrollup_timed_nocenter (iface : interface_state_t) reminders =
          iface with top_timestamp = prev_timestamp
       } in
       let (new_iface, new_reminders) = 
-         handle_selection_change_scroll iface2 reminders
+         handle_selection_change_scroll iface2 reminders false
       in
       (* use a curses scroll operation to shift up the timed window *)
       assert (wscrl new_iface.scr.timed_win (-1));
@@ -398,12 +466,12 @@ let handle_scrollup_untimed (iface : interface_state_t) reminders =
       let new_iface = {
          iface with right_selection = pred iface.right_selection
       } in
-      handle_selection_change new_iface reminders
+      handle_selection_change new_iface reminders new_iface.track_home
    else if iface.top_untimed > 0 then
       let new_iface = {
          iface with top_untimed = pred iface.top_untimed
       } in
-      handle_selection_change new_iface reminders
+      handle_selection_change new_iface reminders new_iface.track_home
    else
       (iface, reminders)
 
@@ -430,7 +498,35 @@ let handle_jump (iface : interface_state_t) reminders jump_func =
          iface with top_timestamp = next_ts;
                     top_desc = 0
       } in
-      handle_selection_change new_iface reminders
+      handle_selection_change new_iface reminders false
+
+
+(* handle switching window focus *)
+let handle_switch_focus (iface : interface_state_t) reminders =
+   let new_iface = 
+      match iface.selected_side with
+      |Left  -> {iface with selected_side = Right}
+      |Right -> {iface with selected_side = Left}
+   in
+   handle_selection_change new_iface reminders new_iface.track_home
+
+
+(* handle switching to the current timeslot *)
+let handle_home (iface : interface_state_t) reminders =
+   let curr_time = Unix.localtime ((Unix.time ()) -. (time_inc iface)) in
+   let (rounded_time, _) = Unix.mktime (round_time iface.zoom_level curr_time) in
+   let new_iface = {
+      iface with top_timestamp = 
+                    if !Rcfile.center_cursor then
+                       rounded_time -. (time_inc iface) *. (float_of_int ((iface.scr.tw_lines / 2) - 2))
+                    else
+                       rounded_time -. (time_inc iface) *. 1.;
+                 top_desc        = 0;
+                 selected_side   = Left;
+                 left_selection  = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 2;
+                 right_selection = 1 
+   } in
+   handle_selection_change new_iface reminders true
 
 
 (* handle a zoom keypress *)
@@ -466,38 +562,14 @@ let handle_zoom (iface : interface_state_t) reminders =
                        zoom_level    = Hour
          }
    in
-   let new_iface = draw_timed new_iface reminders.Remind.all_timed in
-   draw_date_strip new_iface;
-   (new_iface, reminders)
+   if new_iface.track_home then
+      handle_home new_iface reminders
+   else 
+      let new_iface = draw_timed new_iface reminders.Remind.all_timed in
+      draw_date_strip new_iface;
+      (new_iface, reminders)
 
 
-(* handle switching window focus *)
-let handle_switch_focus (iface : interface_state_t) reminders =
-   let new_iface = 
-      match iface.selected_side with
-      |Left  -> {iface with selected_side = Right}
-      |Right -> {iface with selected_side = Left}
-   in
-   handle_selection_change new_iface reminders
-
-
-(* handle switching to the current timeslot *)
-let handle_home (iface : interface_state_t) reminders =
-   let curr_time = Unix.localtime ((Unix.time ()) -. (time_inc iface)) in
-   let (rounded_time, _) = Unix.mktime (round_time iface.zoom_level curr_time) in
-   let new_iface = {
-      iface with top_timestamp = 
-                    if !Rcfile.center_cursor then
-                       rounded_time -. (time_inc iface) *. (float_of_int ((iface.scr.tw_lines / 2) - 2))
-                    else
-                       rounded_time -. (time_inc iface) *. 1.;
-                 top_desc        = 0;
-                 selected_side   = Left;
-                 left_selection  = if !Rcfile.center_cursor then (iface.scr.tw_lines / 2) - 1 else 2;
-                 right_selection = 1
-   } in
-   handle_selection_change new_iface reminders
-
 
 (* handle creation of a new timed or untimed reminder *)
 let handle_new_reminder (iface : interface_state_t) reminders rem_type 
@@ -668,7 +740,7 @@ let handle_find_next (iface : interface_state_t) reminders override_regex =
                assert (doupdate ());
                (iface, reminders)
             |Some (timed_match_ts, timed_match_iface) ->
-               handle_selection_change timed_match_iface new_reminders
+               handle_selection_change timed_match_iface new_reminders false
             end
          |urem :: tail ->
             begin try
@@ -716,12 +788,12 @@ let handle_find_next (iface : interface_state_t) reminders override_regex =
                   (* choose between the timed reminder match and the untimed reminder match *)
                   begin match timed_match with
                   |None ->
-                     handle_selection_change new_iface new_reminders
+                     handle_selection_change new_iface new_reminders false
                   |Some (timed_match_ts, timed_match_iface) ->
                      if timed_match_ts < urem.Remind.ur_start then
-                        handle_selection_change timed_match_iface new_reminders
+                        handle_selection_change timed_match_iface new_reminders false
                      else
-                        handle_selection_change new_iface new_reminders
+                        handle_selection_change new_iface new_reminders false
                   end
                else
                   raise Not_found
@@ -966,7 +1038,7 @@ let do_selection_dialog (iface : interface_state_t) (title : string)
 let handle_edit (iface : interface_state_t) reminders =
    let adjust_s len s =
       let pad = s ^ (String.make len ' ') in
-      Str.string_before pad len
+      Utility.utf8_string_before pad len
    in
    let fl =
       match iface.selected_side with
@@ -1085,7 +1157,7 @@ let handle_scroll_desc_down iface reminders =
 let handle_copy_reminder_aux iface reminders copy_only =
    let adjust_s len s =
       let pad = s ^ (String.make len ' ') in
-      Str.string_before pad len
+      Utility.utf8_string_before pad len
    in
    let fl =
       match iface.selected_side with
@@ -1388,7 +1460,7 @@ let handle_goto (iface : interface_state_t) reminders =
                     entry_mode       = Normal;
                     extended_input   = ""
       } in
-      handle_selection_change new_iface reminders
+      handle_selection_change new_iface reminders false
    end
          
 
@@ -1407,43 +1479,7 @@ let handle_begin_goto (iface : interface_state_t) reminders =
 (* Handle creation of a quick event *)
 let handle_quick_event (iface : interface_state_t) reminders remfile =
    try
-      let (rem_spec, description) = 
-         Time_lang.parse_natural_language_event iface.extended_input
-      in
-      let remline =
-         begin match rem_spec with
-         | Time_lang.Timed (start, finish) ->
-            let date_s = 
-               Printf.sprintf "REM %s %d %d " (Utility.string_of_tm_mon start.Unix.tm_mon)
-               start.Unix.tm_mday (start.Unix.tm_year + 1900)
-            in
-            let start_s = 
-               Printf.sprintf "AT %.2d:%.2d " start.Unix.tm_hour start.Unix.tm_min
-            in
-            let duration_s =
-               if start <> finish then
-                  let (start_ts, _)  = Unix.mktime start
-                  and (finish_ts, _) = Unix.mktime finish in
-                  let dur = int_of_float ((finish_ts -. start_ts) /. 60.0) in
-                  Printf.sprintf "DURATION %d:%.2d " (dur / 60) (dur mod 60)
-               else
-                  ""
-            in
-            date_s ^ start_s ^ duration_s ^ "MSG " ^ description ^ "\n"
-         | Time_lang.Untimed timespec ->
-            let date_s = 
-               Printf.sprintf "REM %s %d %d " (Utility.string_of_tm_mon timespec.Unix.tm_mon)
-               timespec.Unix.tm_mday (timespec.Unix.tm_year + 1900)
-            in
-            date_s ^ "MSG " ^ description ^ "\n"
-         end
-      in
-      (* append the remline to the reminders file *)
-      let remfile_channel = 
-         open_out_gen [Open_append; Open_creat; Open_text] 416 remfile
-      in
-      output_string remfile_channel remline;
-      close_out remfile_channel;
+      let (rem_spec, remline) = append_quick_event iface.extended_input remfile in
       (* navigate to the new reminder *)
       let (is_timed, start) =
          match rem_spec with
@@ -1473,7 +1509,7 @@ let handle_quick_event (iface : interface_state_t) reminders remfile =
                     entry_mode      = Normal;
                     extended_input  = ""
       } in
-      handle_selection_change new_iface r
+      handle_selection_change new_iface r false
    with Time_lang.Event_parse_error s ->
       failwith s
 
@@ -1707,11 +1743,11 @@ let handle_keypress key (iface : interface_state_t) reminders =
                end
             end
          |Rcfile.EntryBackspace ->
-            let len = String.length iface.extended_input in
+            let len = Utility.utf8_len iface.extended_input in
             if len > 0 then begin
                let new_iface = {
                   iface with extended_input =
-                             Str.string_before iface.extended_input (pred len)
+                             Utility.utf8_string_before iface.extended_input (pred len)
                } in
                let prompt =
                   match ext_mode with
@@ -1808,15 +1844,24 @@ let rec do_main_loop (iface : interface_state_t) reminders last_update =
       (* refresh the msg window (which contains a clock)
        * every wgetch timeout cycle *)
       let iface = draw_msg iface in
-      assert (doupdate ());
+      begin match iface.resize_failed_win with
+      | None   -> assert (doupdate ())
+      | Some w -> ()
+      end;
       let key = wgetch iface.scr.help_win in
       let new_iface, new_reminders = 
          (* key = -1 is ncurses wgetch timeout error *)
          if key <> ~- 1 then
             if key = Key.resize then
                handle_resize iface reminders
-            else
-               handle_keypress key iface reminders
+            else 
+               begin match iface.resize_failed_win with
+               | None   -> handle_keypress key iface reminders
+               | Some w -> let _ = beep () in (iface, reminders)
+               end
+         else if iface.track_home then
+            (* cursor tracks the current time if requested *)
+            handle_home iface reminders
          else
             (iface, reminders)
       in
