@@ -1,5 +1,5 @@
 (*  Wyrd -- a curses-based front-end for Remind
- *  Copyright (C) 2005, 2006 Paul Pelzl
+ *  Copyright (C) 2005, 2006, 2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, Version 2,
@@ -77,7 +77,7 @@ let table_command_commandstr = Hashtbl.create 30
 (* Default Remind command *)
 let remind_command = ref "remind"
 (* Default reminders file *)
-let reminders_file = ref "~/.reminders"
+let reminders_file = ref "$HOME/.reminders"
 (* Default editing command strings *)
 let edit_old_command = ref "vim +%n %f"
 let edit_new_command = ref "vim -c '$' %f"
@@ -119,6 +119,10 @@ let goto_big_endian = ref true
 let quick_date_US = ref true
 (* print week numbers? *)
 let number_weeks = ref false
+(* home is "sticky"? *)
+let home_sticky = ref true
+(* width of calendar/untimed reminder windows *)
+let untimed_window_width = ref 40
 (* List of included rc files *)
 let included_rcfiles : (string list) ref = ref []
 
@@ -545,6 +549,10 @@ let parse_line line_stream =
          parse_set "quick_date_US" quick_date_US bool_of_string "Expected a boolean string after "
       | [< 'Ident "number_weeks" >] ->
          parse_set "number_weeks" number_weeks bool_of_string "Expected a boolean string after "
+      | [< 'Ident "home_sticky" >] ->
+         parse_set "home_sticky" home_sticky bool_of_string "Expected a boolean string after "
+      | [< 'Ident "untimed_window_width" >] ->
+         parse_set "untimed_window_width" untimed_window_width int_of_string "Expected an integral string after "
       | [< >] ->
          config_failwith ("Unmatched variable name after \"set\"")
       end
